module: Testsuite
author: Michael Erdmann (michael.erdmann@snafu.de)
synopsis: This package provides a framework for test.
copyright: see below

//****************************************************************************
//                                                                          **
// This is free software; you can redistribute it and/or modify it under    **
// terms of the  GNU General Public License as published  by the Free Soft- **
// ware  Foundation;  either version 2,  or (at your option) any later ver- **
// sion. It is distributed in the hope that it will be useful, but WITH OUT **
// ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY     **
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License **
// for more details.  							    **
// You should have received a copy of the GNU General Public License        **
// distributed with this software; see file COPYING. If not, write to the   **
// Free Software Foundation,  59 Temple Place - Suite 330,  Boston,         **
// MA 02111-1307, USA.                                                      **
//                                                                          **
// As a special exception, if other files instantiate generics from this    **
// unit, or you link  this unit with other files to produce an executable,  **
// this  unit  does not  by itself cause  the resulting  executable to be   **
// covered  by the  GNU  General  Public  License. This exception does not  **
// however invalidate  any other reasons why  the executable file might be  **
// covered by the  GNU Public License.                                      **
//                                                                          **
//                                                                          **
//****************************************************************************

define open class <testsuite-data> ( <object> )
   slot name, init-keyword: name: ;
   slot testcases, init-keyword: testcases: ;
   slot init;
   slot cleanup;
end class;

define variable *testcases* = make ( <string-table> );

/*
 * this macro defines a single test case
 */
define macro testcase-definer
   { define testcase ?:name prepare ?pbody:* execute ?ebody:* verdict ?vbody:* end } =>
   { *testcases*[ ?"name" ] := method ()  
     	   ?pbody 

	   block() ?ebody ?vbody end;
        end ;
   }
end macro ;

/*
 * Create a test suite by copying all collected test cases into the test suite. 
 */
define method  make-testsuite( name :: <string>, #key init = #f, cleanup = #f ) 
                                                        => (result :: <testsuite-data>)
   let result = make( <testsuite-data>, name: name, testcases: *testcases* );

   result.init := init;
   result.cleanup := cleanup;

   *testcases* := make( <string-table> );

   result;
end method;

/*
 * Execute the given test suite.
 */									                      
define method execute-suite( suite :: <testsuite-data> ) => ( result :: <string-table> )
   let result = make( <string-table> );
   let success  = 0;
   let faulty   = 0;
   let executed = 0;

   format-out("============================================================================\n");
   format-out("Executing Testsuite: %s\n", suite.name );   
   format-out("\n");

   block () 
      if( suite.init )
         suite.init();
      end if;
                                                                                                
      for( e keyed-by index in suite.testcases )
        format-out("*** Executing: %s\n", index );
         block() 
            result[ index ] := e();
	 exception ( ex :: <condition> )
	    result[ index ] := #f ;
	    format-out("Exception %=\n", ex );
	    //format-out(condition-format-string(ex), condition-format-arguments(ex));
	 end ;
	 format-out("\n");
      end;

      if( suite.cleanup )
         suite.cleanup;
      end if;
   end ;

   format-out("Execution finished.\n\n");

   for( e keyed-by index in result )
      let status = result[ index ];

      executed := executed + 1;

      if( ~status )
         faulty := faulty + 1;
         format-out( "%s failed\n", index );
      else
         success := success + 1;
         format-out( "%s passed successfully\n", index );
      end if;
   end;

   format-out("\nSummary:\n" );
   format-out("     Nnumber of executed tests   : %d\n", executed );
   format-out("     Nnumber of successfull tests: %d\n", success );
   format-out("     Nnumber of faulty tests     : %d\n", faulty  );
   format-out("\n" );

   result;
end method;

